import {__decorate,__param}from'tslib';import {IDisposable}from'@aster-js/core';import {ServiceIdentifier,ServiceContract,IoCKernel,ServiceDescriptor}from'@aster-js/ioc';import {html,unsafeCSS,LitElement,css}from'lit';import {property,customElement}from'lit/decorators.js';import {Tooltip}from'@aster-elements/tooltip';import {classMap}from'lit/directives/class-map.js';import*as moment from'moment';import {unsafeSVG}from'lit/directives/unsafe-svg.js';const ITooltipService = ServiceIdentifier("ITooltipService");const ITimelineRenderer = ServiceIdentifier("ITimelineRenderer");const ITimelineStepRenderer = ServiceIdentifier("ITimelineStepRenderer");let AsterTooltipService = class AsterTooltipService {
    constructor(_tooltipName) {
        this._tooltipName = _tooltipName;
    }
    show(ev, content) {
        const tooltip = Tooltip.get(this._tooltipName);
        tooltip.show(ev, content, { autoHide: true, align: "center", position: "bottom" });
    }
};
AsterTooltipService = __decorate([
    ServiceContract(ITooltipService)
], AsterTooltipService);let DefaultTimelineRenderer = class DefaultTimelineRenderer {
    constructor(_stepRenderer) {
        this._stepRenderer = _stepRenderer;
    }
    *render(items, opts) {
        yield* items.map((x, i) => this.renderItem(x, i, items, opts), this).flatMap(x => [...x]);
    }
    *renderItem(item, index, items, opts) {
        if (index) {
            yield html `${this._stepRenderer.renderLead(item, index, items, opts)}`;
        }
        yield html `${this._stepRenderer.renderDetail(item, index, items, opts)}`;
        yield html `${this._stepRenderer.renderStep(item, index, items, opts)}`;
    }
};
DefaultTimelineRenderer = __decorate([
    ServiceContract(ITimelineRenderer),
    __param(0, ITimelineStepRenderer)
], DefaultTimelineRenderer);class TimelineStepRenderer {
    constructor(_toolTipService) {
        this._toolTipService = _toolTipService;
    }
    renderLead(item, index, items, opts) {
        const leadStyle = this.resolveLeadLayout(index, opts.layout);
        const duration = this.resolveLeadTime(items[index - 1], item, opts);
        return html `<aster-timeline-lead style=${leadStyle} .layout=${opts.layout} .duration=${duration} .alternate=${index % 2 !== 0}></aster-timeline-lead>`;
    }
    renderDetail(item, index, items, opts) {
        const blockquoteStyle = this.resolveBlockquoteLayout(index, opts.layout);
        const detailStyle = this.resolveDetailLayout(index, opts.layout);
        const detail = this.renderDetailContent(item, index, items, opts);
        if (detail) {
            return html `
            <aster-timeline-blockquote style=${blockquoteStyle} .layout=${opts.layout}></aster-timeline-blockquote>
            <aster-timeline-detail style=${detailStyle} .layout=${opts.layout}>
                ${detail}
            </aster-timeline-detail>`;
        }
    }
    renderStep(item, index, items, opts) {
        const stepColumn = this.resolveStepLayout(index, opts.layout);
        return html `<aster-timeline-step
            style=${stepColumn}
            .layout=${opts.layout}
            @mouseenter="${(ev) => this.onDidMouseEnterStep(ev, item, index, items, opts)}">
            ${this.renderStepContent(item, index, items, opts)}
        </aster-timeline-step>`;
    }
    onDidMouseEnterStep(ev, item, index, items, opts) {
        const content = this.renderTooltipContent(item, index, items, opts);
        if (content) {
            this._toolTipService.show(ev, content);
        }
    }
    resolveLeadLayout(index, layout) {
        const leadColumn = (index - 1) * 2 + 2;
        if (layout == "horizontal") {
            return `grid-column: ${leadColumn}`;
        }
        return `grid-row: ${leadColumn}`;
    }
    resolveBlockquoteLayout(index, layout) {
        if (layout == "horizontal") {
            return `grid-column:${index * 2 + 1}; grid-row:${index % 2 === 0 ? 1 : 3}`;
        }
        return `grid-row:${index * 2 + 1}; grid-column:${index % 2 === 0 ? 1 : 3}`;
    }
    resolveDetailLayout(index, layout) {
        if (layout == "horizontal") {
            return `grid-column:${index * 2 + 2}/span 2; grid-row:${index % 2 === 0 ? 1 : 3}`;
        }
        return `grid-row:${index * 2 + 2}/span 2;grid-column:${index % 2 === 0 ? 1 : 3}`;
    }
    resolveStepLayout(index, layout) {
        if (layout == "horizontal") {
            return `grid-column:${index * 2 + 1}`;
        }
        return `grid-row:${index * 2 + 1}`;
    }
}let DefaultTimelineStepRenderer = class DefaultTimelineStepRenderer extends TimelineStepRenderer {
    constructor(toolTipService) { super(toolTipService); }
    resolveLeadTime(first, second, opts) {
        return second.date.getTime() - first.date.getTime();
    }
    renderDetailContent(item, index, items, _opts) {
        return this.resolveTemplateResult(item.detail, item, index, items);
    }
    renderStepContent(item, index, items, _opts) {
        return this.resolveTemplateResult(item.icon, item, index, items);
    }
    renderTooltipContent(item, index, items, _opts) {
        return this.resolveTemplateResult(item.tooltip, item, index, items);
    }
    resolveTemplateResult(value, item, index, items) {
        if (typeof value === "function") {
            return value(item, index, items);
        }
        return html `${value}`;
    }
};
DefaultTimelineStepRenderer = __decorate([
    ServiceContract(ITimelineStepRenderer),
    __param(0, ITooltipService)
], DefaultTimelineStepRenderer);var styles$4 = ":host {\n    --_aster-timeline__border-color: var(--aster-timeline__border-color, #808080);\n    --_aster-timeline__step-icon-size: var(--aster-timeline__step-icon-size, 26px);\n    --_aster-timeline__lead-time-font-size: var(--aster-timeline__lead-time-font-size, 0.8em);\n    display: block;\n    box-sizing: border-box;\n}\n\n.container {\n    justify-items: center;\n    width: 100%;\n}\n\n.container.horizontal {\n    display: grid;\n    grid-template-rows:  minmax(0, 1fr) var(--_aster-timeline__step-icon-size) minmax(0, 1fr);\n    grid-auto-columns: minmax(0, max-content) minmax(0, 1fr);\n}\n\n.container.vertical {\n    display: grid;\n    gap: 0 10px;\n    grid-auto-rows: minmax(0, max-content) minmax(0, 1fr);\n    grid-template-columns: minmax(0, 1fr) var(--_aster-timeline__step-icon-size) minmax(0, 1fr);\n}\n\n.horizontal > aster-timeline-lead {\n    grid-row: 2;\n    width: 100%;\n}\n\n.vertical > aster-timeline-lead {\n    grid-column: 1 / span 3;\n    height: 100%;\n}\n\n.horizontal > aster-timeline-step {\n    grid-row: 2;\n    align-self: center;\n}\n\n.vertical > aster-timeline-step {\n    grid-column: 2;\n    align-self: center;\n}\n";let Timeline = class Timeline extends LitElement {
    constructor() {
        super(...arguments);
        this._module = null;
        this.layout = "horizontal";
        this.items = [];
    }
    init(iocModule) {
        const scope = iocModule ? iocModule.createChildScope("grid") : IoCKernel.create();
        this._module = scope.configure(s => this.configure(s)).build();
    }
    configure(services) {
        this.dispatchEvent(new CustomEvent("configure-services", { bubbles: true, composed: true, detail: services }));
        this.tryAddDefaultImpl(services, DefaultTimelineStepRenderer);
        this.tryAddDefaultImpl(services, DefaultTimelineRenderer);
        this.tryAddDefaultImpl(services, AsterTooltipService);
    }
    tryAddDefaultImpl(services, ctor) {
        const serviceId = ServiceContract.resolve(ctor);
        if (serviceId && !services.has(serviceId)) {
            const desc = new ServiceDescriptor(serviceId, 1 /* ServiceLifetime.scoped */, 1 /* ServiceScope.container */, ctor, [], false);
            services.add(desc);
        }
    }
    update(changedProperties) {
        if (changedProperties.has("layout") || changedProperties.has("manualInit")) {
            IDisposable.safeDispose(this._module);
            this._module = null;
        }
        super.update(changedProperties);
    }
    render() {
        return html `<div class="container ${this.layout}">${this.renderSteps()}</div>`;
    }
    *renderSteps() {
        if (this.items.length) {
            if (!this._module && !this.manualInit) {
                this.init();
            }
            if (this._module) {
                const renderer = this._module.services.get(ITimelineRenderer, true);
                yield* renderer.render(this.items, { layout: this.layout });
            }
        }
    }
};
Timeline.styles = unsafeCSS(styles$4);
__decorate([
    property({ type: String })
], Timeline.prototype, "layout", void 0);
__decorate([
    property()
], Timeline.prototype, "items", void 0);
__decorate([
    property({ type: Boolean, attribute: "manual-init" })
], Timeline.prototype, "manualInit", void 0);
Timeline = __decorate([
    customElement("aster-timeline")
], Timeline);var styles$3 = ":host {\n    display: block;\n    font-size: var(--_aster-timeline__lead-time-font-size);\n}\n\n.container {\n    display: grid;\n    justify-items: center;\n    align-items: center;\n    height: 100%;\n}\n.line {\n    grid-area: lead;\n    background-color: var(--_aster-timeline__border-color);\n}\n.indicator {\n    grid-area: lead;\n    height: 6px;\n    width: 6px;\n    border-radius: 50%;\n    background-color: var(--_aster-timeline__border-color);\n}\n.duration {\n    grid-area: duration;\n}\n\n.container.horizontal {\n    grid-template-rows: minmax(0, 1fr) var(--_aster-timeline__step-icon-size) minmax(0, 1fr);\n    grid-template-areas: \"duration\" \"lead\" \"_\"\n}\n.container.horizontal.alternate{\n    grid-template-areas: \"_\" \"lead\" \"duration\"\n}\n.horizontal .line {\n    height: 3px;\n    width: 100%;\n}\n\n.container.vertical {\n    grid-template-columns: minmax(0, 1fr) var(--_aster-timeline__step-icon-size) minmax(0, 1fr);\n    grid-template-areas: \"duration lead _\"\n}\n.container.vertical.alternate{\n    grid-template-areas: \"_ lead duration\"\n}\n.vertical .line {\n    width: 3px;\n    height: 100%;\n}\n.vertical .duration {\n    justify-self: flex-end;\n}\n.vertical.alternate .duration {\n    justify-self: flex-start;\n}\n";let TimelineLead = class TimelineLead extends LitElement {
    constructor() {
        super(...arguments);
        /** Lead time in milliseconds */
        this.duration = 0;
        this.layout = "horizontal";
        this.alternate = false;
    }
    render() {
        const timespan = this.duration > 0 ? moment.duration(this.duration).humanize() : html `&nbsp;`;
        const classes = {
            container: true,
            [this.layout]: true,
            alternate: this.alternate
        };
        return html `<div class="${classMap(classes)}">
            <div class="line"></div>
            <div class="indicator"></div>
            <div class="duration">${timespan}</div>
        </div>`;
    }
};
TimelineLead.styles = unsafeCSS(styles$3);
__decorate([
    property({ type: Number })
], TimelineLead.prototype, "duration", void 0);
__decorate([
    property({ type: String })
], TimelineLead.prototype, "layout", void 0);
__decorate([
    property({ type: Boolean })
], TimelineLead.prototype, "alternate", void 0);
TimelineLead = __decorate([
    customElement("aster-timeline-lead")
], TimelineLead);var styles$2 = ":host {\n    display: block;\n    width: fit-content;\n}\n\n#container {\n    width: 100%;\n    height: 100%;\n}\n";let TimelineStep = class TimelineStep extends LitElement {
    constructor() {
        super(...arguments);
        this.layout = "horizontal";
    }
    render() {
        return html `<div id="container ${this.layout}"><slot></slot></div>`;
    }
};
TimelineStep.styles = unsafeCSS(styles$2);
__decorate([
    property({ type: Number })
], TimelineStep.prototype, "layout", void 0);
TimelineStep = __decorate([
    customElement("aster-timeline-step")
], TimelineStep);var styles$1 = ":host {\n    display: block;\n}\n.circle {\n    border-radius: 50%;\n    width:25px;\n    height:25px;\n}\n.rounded {\n    border-radius: 5px;\n    width:25px;\n    height:25px;\n}\n\n.none {\n    border: 4px solid rgb(137 213 137);\n    background-color: #2a852a;\n}\n\n.progress {\n    border: 4px solid rgb(137 213 137);\n    background-color: #2a852a;\n}\n\n.info {\n    border: 4px solid rgb(99, 154, 241);\n    background-color: #1952bb;\n}\n\n.ask {\n    border: 4px solid rgb(96, 235, 154);\n    background-color: #209aaa;\n}\n\n.ok {\n    border: 4px solid rgb(137 213 137);\n    background-color: #2a852a;\n}\n\n.warn {\n    border: 4px solid rgb(206, 214, 206);\n    background-color: #7f867f;\n}\n\n.error {\n    border: 4px solid rgb(137 213 137);\n    background-color: #2a852a;\n}\n\nsvg {\n    color: #fff;\n    width: 18px;\n    height: 18px;\n    padding: 3px;\n}\n";let TimelineStepIcon = class TimelineStepIcon extends LitElement {
    constructor() {
        super(...arguments);
        this.type = "none";
        this.shape = "circle";
    }
    render() {
        return html `<div class="${this.type} ${this.shape}">${unsafeSVG(`<svg><use xlink:href="${this.svg}" /></svg>`)}</div>`;
    }
};
TimelineStepIcon.styles = unsafeCSS(styles$1);
__decorate([
    property({ type: String })
], TimelineStepIcon.prototype, "svg", void 0);
__decorate([
    property({ type: String })
], TimelineStepIcon.prototype, "type", void 0);
__decorate([
    property({ type: String })
], TimelineStepIcon.prototype, "shape", void 0);
TimelineStepIcon = __decorate([
    customElement("aster-timeline-step-icon")
], TimelineStepIcon);var styles = ":host {\n    display: block;\n    padding: 5px;\n}\n";let TimelineDetail = class TimelineDetail extends LitElement {
    constructor() {
        super(...arguments);
        this.layout = "horizontal";
    }
    render() {
        return html `<div class=${this.layout}><slot></slot></div>`;
    }
};
TimelineDetail.styles = unsafeCSS(styles);
__decorate([
    property({ type: Number })
], TimelineDetail.prototype, "layout", void 0);
TimelineDetail = __decorate([
    customElement("aster-timeline-detail")
], TimelineDetail);let TimelineBlockquote = class TimelineBlockquote extends LitElement {
    constructor() {
        super(...arguments);
        this.layout = "horizontal";
    }
    render() {
        return html `<div class="container"><div class="line ${this.layout}"></div></div>`;
    }
};
TimelineBlockquote.styles = css `
    :host {
        display: block;
    }
    .container {
        display: grid;
        width: 100%;
        height: 100%;
        justify-items: center;
        align-items: center;
    }
    .line.horizontal {
        width: 0;
        height: 100%;
        border-left: 1px dotted var(--_aster-timeline__border-color);
    }
    .line.vertical {
        width: 100%;
        height: 0;
        border-top: 1px dotted var(--_aster-timeline__border-color);
    }`;
__decorate([
    property({ type: Number })
], TimelineBlockquote.prototype, "layout", void 0);
TimelineBlockquote = __decorate([
    customElement("aster-timeline-blockquote")
], TimelineBlockquote);export{AsterTooltipService,DefaultTimelineRenderer,DefaultTimelineStepRenderer,ITimelineRenderer,ITimelineStepRenderer,ITooltipService,Timeline,TimelineBlockquote,TimelineDetail,TimelineLead,TimelineStep,TimelineStepIcon,TimelineStepRenderer};//# sourceMappingURL=aster-timeline.js.map
